<?php
  // XML::MODUL   (Class CGameserver)
  // XML::VERSION (0.035)
  // XML::AUTHOR  (Henrik Schack Jensen, Extensions by ::[CTH]Goldenhawk::)
  // XML::INFO    (Dieses Klasse verwaltet die Informationen, die ein Gameserver liefert und beinhaltet CUnreal, CQuake3 und CCounterStrike)

  // XML::COMMENT (Vesrion, Name und Autor festlege)
  $VERSION = "0.035";
  $NAME    = "CGameserver";
  $AUTHOR  = "::[CTH]Goldenhawk::";

  // XML::COMMENT (Klasse CUnreal festlegen)
  class CUnreal
  {
	// XML::COMMENT (Info about players)
	var $m_playerinfo  =  "";
	var $m_servervars  =  "";
	// XML::COMMENT (Info about the server)
	var $errmsg		   ="";

    // XML::FUNC::timenow ($self->timenow())
    // XML::DESC::timenow (Liefert die genaue Zeit fr das Timeout zurck)
    // XML::VERSION::timenow  (1.0)
    // XML::RETURN::timenow ($doubleval)
	function timenow()
	{
	  return doubleval(ereg_replace('^0\.([0-9]*) ([0-9]*)$','\\2.\\1',microtime()));
	}

    // XML::FUNC::getServerData ($self->getServerData($m_command,$m_serveraddress,$m_port,$m_waittime))
    // XML::DESC::getServerData (Liest Daten vom Server)
    // XML::VERSION::getServerData  (1.0)
    // XML::RETURN::getServerData ($string)
	function getServerData($m_command,$m_serveraddress,$m_port,$m_waittime)
	{
	  $serverdata  ="";
	  $serverdatalen = 0;

	  if ($m_waittime< 500)
	    $m_waittime= 500;

	  if ($m_waittime>2000)
	    $m_waittime=2000;

	  $m_waittime = doubleval($m_waittime/1000.0);

	  if (!$cssocket=fsockopen("udp://".$m_serveraddress,$m_port+1,$errnr))
	  {
	    $this->errmsg="No connection";
	    return "";
	  }

	  socket_set_blocking($cssocket,true);
	  // XML::CODE (socket_set_timeout($cssocket,0,500000);)
	  fwrite($cssocket,$m_command,strlen($m_command));

	  // XML::COMMENT (Mark)
	  $starttime=$this->timenow();
	  do
	  {
	    $serverdata.=fgetc($cssocket);
		$serverdatalen++;
		$socketstatus=socket_get_status($cssocket);
		if ($this->timenow()>($starttime+$m_waittime))
		{
		  $this->errmsg="Connection timed out";
		  fclose($cssocket);
		  return "";
		}
	  } while (substr($serverdata,strlen($serverdata)-7)!="\\final\\");

	  fclose($cssocket);
	  return $serverdata;
	}


    // XML::FUNC::getServerStatus ($self->getServerStatus($m_serveraddress,$m_port,$m_waittime))
    // XML::DESC::getServerStatus (Liest Regeln und Servervariablen)
    // XML::VERSION::getServerStatus  (1.0)
    // XML::RETURN::getServerStatus (BOOL|$m_servervars|$m_playerinfo)
	function getServerStatus($m_serveraddress,$m_port,$m_waittime)
	{
	  $cmd = "\\status\\";
	  $serverdata = $this->getServerData($cmd,$m_serveraddress,$m_port,$m_waittime);

	  // XML::COMMENT (Is data complete ?)
	  // XML::COMMENT (This strange doublecheck is needed since UT serverdata sometimes arrives out of order)
	  if ((substr($serverdata,strlen($serverdata)-7)!="\\final\\") || (substr($serverdata,1,8)!="gamename"))
	  {
	    $this->errmsg="Data incomplete";
		return false;
	  }

	  // XML::COMMENT (Remove \\final\\ text at end of data)
	  $serverdata=substr($serverdata,1,strlen($serverdata)-7);

	  // XML::COMMENT (Split data and fill into array)
	  $name_tok = strtok ($serverdata,"\\");
	  $val_tok  = strtok ("\\");

	  while (strlen($name_tok))
	  {
	    if ($name_tok!="queryid")
	    {
		  $vars[$name_tok]=$val_tok;
		}
		$name_tok = strtok ("\\");
		$val_tok  = strtok ("\\");
	  }

	  while (list($key,$data) = each ($vars))
	  {
	    if (!strncmp($key,"player_",7))
	    {
		  $player[substr($key,7)]["name"]=$data;
		}
		elseif (!strncmp($key,"ping_",5))
		{
		  $player[substr($key,5)]["ping"]=$data;
		}
		elseif (!strncmp($key,"frags_",6))
		{
		  $player[substr($key,6)]["frags"]=$data;
		}
		elseif (!strncmp($key,"team_",5))
		{
		  $player[substr($key,5)]["team"]=$data;
		}
		elseif (!strncmp($key,"skin_",5))
		{
		  $player[substr($key,5)]["skin"]=$data;
		}
		elseif (!strncmp($key,"mesh_",5))
		{
		  $player[substr($key,5)]["mesh"]=$data;
		}
		elseif (!strncmp($key,"face_",5))
		{
		  $player[substr($key,5)]["face"]=$data;
		}
		elseif (!strncmp($key,"ngsecret_",9))
		{
		  $player[substr($key,9)]["ngsecret"]=$data;
		}
		else
		{
		  $this->m_servervars[$key]=$data;
		}
	  }

	  // XML::COMMENT (Move playerinfo to membervariable)

	  for ($i=0;$i<$this->m_servervars["numplayers"];$i++)
	  {
	    $this->m_playerinfo[$i]=array("name"=>$player[$i]["name"],"ping"=>$player[$i]["ping"],"frags"=>$player[$i]["frags"]);
	  }

	  // XML::COMMENT (If there are players on the server we sort them by frags)
	  if ($this->m_servervars["numplayers"]>0)
	    usort($this->m_playerinfo,"fragsort");

	  // XML::COMMENT (Make maptitle lowercase)
	  $this->m_servervars["mapname"]=strtolower($this->m_servervars["mapname"]);

	  return true;
	}

    // XML::FUNC::convertToStandard ($self->convertToStandard())
    // XML::DESC::convertToStandard (Konvertiert alle Variablen, die in $m_serverinfo gespeichert sind, zu einem gemeinsamen Standard)
    // XML::VERSION::convertToStandard  (1.0)
    // XML::RETURN::convertToStandard (BOOL|$m_srvinfo)
	function convertToStandard()
	{
	  $this->m_srvinfo['iCurrentClients'] = $this->m_servervars['numplayers'];
	  $this->m_srvinfo['szMapName'] 	  = $this->m_servervars['mapname'];
	  $this->m_srvinfo['szHostName'] 	  = $this->m_servervars['hostname'];
	  $this->m_srvinfo['iTimelimit'] 	  = $this->m_servervars['timelimit'];
	  $this->m_srvinfo['iFraglimit'] 	  = $this->m_servervars['fraglimit'];
	  $this->m_srvinfo['iMaxClients'] 	  = $this->m_servervars['maxplayers'];
	  $this->m_srvinfo['szVersion'] 	  = $this->m_servervars['gamever'];
	  $this->m_srvinfo['szGameType'] 	  = $this->m_servervars['gametype']." - ".$this->m_servervars['gamestyle'];
	  // XML::COMMENT (Passwortgeschtzt?)
	  ($this->m_servervars['password'] = 'False') ? ($this->m_srvinfo['iPWProtected'] = '0') : ($this->m_srvinfo['iPWProtected'] = 1);
	  ($this->m_servervars['listenserver'] = 'False') ? ($this->m_srvinfo['szServerType'] = 'Dedicated') : ($this->m_srvinfo['szServerType'] = 'Listenserver');
	  $this->m_srvinfo['szServerOS'] 	  = "Unknown (".$this->m_srvinfo['szServerType'].")";

	  return true;
	}
  }

  // XML::COMMENT (Klasse CQuake3 festlegen)
  class CQuake3
  {
	// XML::COMMENT (Info about players)
	var $m_playerinfo = "";
	// XML::COMMENT (Info about the server)
	var $m_servervars = "";

    // XML::FUNC::timenow ($self->timenow())
    // XML::DESC::timenow (Liefert die genaue Zeit fr das Timeout zurck)
    // XML::VERSION::timenow  (1.0)
    // XML::RETURN::timenow ($doubleval)
	function timenow()
	{
	  return doubleval(ereg_replace('^0\.([0-9]*) ([0-9]*)$','\\2.\\1',microtime()));
	}

    // XML::FUNC::removeFunChars ($self->removeFunChars($string))
    // XML::DESC::removeFunChars (Entfernt spezielle Zeichen aus dem String)
    // XML::VERSION::removeFunChars  (1.0)
    // XML::RETURN::removeFunChars ($string)
	function removeFunChars($data)
	{
	  $result = "";
	  $skipnextchar = false;

	  for ($i=0;$i<strlen($data);$i++)
	  {
	    if (!$skipnextchar)
	    {
		  $currentchar = ord(substr($data,$i,1));
		  if (($currentchar == 27) || ($currentchar == 94))
		  {
		    $skipnextchar = true;
		  }
		  else
		  {
		    if (($currentchar >= 32) && ($currentchar <= 127))
		      $result = $result.chr($currentchar);

			if (($currentchar>=160) && ($currentchar <= 255))
			  $result = $result.chr($currentchar-128);
		  }
		}
		else
		{
		  $skipnextchar = false;
		}
	  }
	  return $result;
	}

    // XML::FUNC::getServerData ($self->getServerData($m_command,$m_serveraddress,$m_port,$m_waittime))
    // XML::DESC::getServerData (Liefert Daten vom Server zurck)
    // XML::VERSION::getServerData  (1.0)
    // XML::RETURN::getServerData ($string)
	function getServerData($m_command,$m_serveraddress,$m_port,$m_waittime)
	{
	  $serverdata 	 = "";
	  $serverdatalen = 0;

	  if ($m_waittime< 500)
	    $m_waittime= 500;

	  if ($m_waittime>2000)
	    $m_waittime=2000;

	  $m_waittime = doubleval($m_waittime/1000.0);

	  if (!$cssocket=fsockopen("udp://".$m_serveraddress,$m_port,$errnr))
	  {
	    $this->errmsg = "No connection";
		return "";
	  }

	  socket_set_blocking($cssocket,true);
	  // XML::CODE (socket_set_timeout($cssocket,0,500000);)

	  fwrite($cssocket,$m_command,strlen($m_command));
	  // XML::CODE (Mark)
	  $starttime = $this->timenow();
	  do
	  {
	    $serverdata .= fgetc($cssocket);
		$serverdatalen++;
		$socketstatus = socket_get_status($cssocket);

		if ($this->timenow()>($starttime+$m_waittime))
		{
		  $this->errmsg = "Connection timed out";
		  fclose($cssocket);
		  return "";
		}
	  } while ($socketstatus["unread_bytes"] );

	  fclose($cssocket);
	  return $serverdata;
	}


    // XML::FUNC::getServerStatus ($self->getServerStatus($m_serveraddress,$m_port,$m_waittime))
    // XML::DESC::getServerStatus (Liefert Status des Servers zurck)
    // XML::VERSION::getServerStatus (1.0)
    // XML::RETURN::getServerStatus (BOOL|$m_servervars|$m_playerinfo)
	function getServerStatus($m_serveraddress,$m_port,$m_waittime)
	{
	  $cmd="\xFF\xFF\xFF\xFFgetstatus\n";
	  $serverdata = $this->getServerData($cmd,$m_serveraddress,$m_port,$m_waittime);
	  if (strlen($serverdata)>20)
	  {
	    $serverdata = substr($serverdata,20);
	  }
	  else
	  {
	    return false;
	  }

	  $srvvars = substr($serverdata,0,strpos($serverdata,"\n"))."\\";
	  $players = substr($serverdata,strpos($serverdata,"\n")+1);

	  // XML::COMMENT (Skip the last linefeed)
	  $players=substr($players,0,strlen($players)-1);

	  // XML::COMMENT (Split playerinfo and fill into array)
	  $playercount = 0;
	  if (strlen($players))
	  {
	    $playercollection = explode("\n",$players);
		while (list($key,$data) = each ($playercollection))
		{
		  eregi("^([-0-9]+) ([-0-9]+) \"(.*)\"",$data,$player);
		  $this->m_playerinfo[$key]=array("frags"=>$player[1],"time"=>$player[2],"name"=>$this->removeFunChars($player[3]));
		  $playercount++;
		}
	  }
	  // XML::COMMENT (Fake servervar containing number of current players
	  $this->m_servervars["sv_currentclients"] = $playercount;

	  if ($playercount>0)
	    usort($this->m_playerinfo,"fragsort");

	  // XML::COMMENT (Split servervars and fill into array)
	  $name_tok = strtok ($srvvars,"\\");
	  $val_tok  = strtok ("\\");
	  while (strlen($name_tok))
	  {
	    $this->m_servervars[$name_tok] = $val_tok;
		$name_tok = strtok ("\\");
		$val_tok  = strtok ("\\");
	  }

	  return true;
	}

    // XML::FUNC::execCmd ($self->execCmd($m_serveraddress,$m_port,$m_waittime,$in_rconpw,$m_port))
    // XML::DESC::execCmd (Fhrt Kommando aus)
    // XML::VERSION::execCmd  (1.0)
    // XML::RETURN::execCmd (BOOL)
	function execCmd($m_serveraddress,$m_port,$m_waittime,$in_rconpw,$m_command)
	{
	  $serverdata    = "";
	  $serverdatalen = 0;

	  if ($m_waittime< 500)
	    $m_waittime= 500;
	  if ($m_waittime>2000)
	    $m_waittime=2000;

	  $m_waittime = doubleval($m_waittime/1000.0);
	  if (!$cssocket=fsockopen("udp://".$m_serveraddress,$m_port,$errnr))
	  {
	    $this->errmsg = "No connection";
	    return "";
	  }

	  socket_set_blocking($cssocket,true);
	  // XML::CODE (socket_set_timeout($cssocket,0,500000);)

	  fwrite($cssocket, sprintf("%c%c%c%c%s%c", 0xFF, 0xFF, 0xFF, 0xFF, "rcon \"$in_rconpw\" $m_command", 0x00));

      $ans = fread($cssocket,8);
	  (preg_match("/(.*)Ba/i",$ans,$ar)) ? ($ret = 0) : ($ret = 1);

	  if ($ret == 0)
	    return false;
	  else
	    return true;
	}

    // XML::FUNC::convertToStandard ($self->convertToStandard())
    // XML::DESC::convertToStandard (Konvertiert alle Variablen, die in $m_serverinfo gespeichert sind, zu einem gemeinsamen Standard)
    // XML::VERSION::convertToStandard  (1.0)
    // XML::RETURN::convertToStandard (BOOL|$m_srvinfo)
	function convertToStandard()
	{
	  $this->m_srvinfo['iCurrentClients'] = $this->m_servervars['sv_currentclients'];
	  $this->m_srvinfo['szMapName'] 	  = $this->m_servervars['mapname'];
	  $this->m_srvinfo['szHostName'] 	  = $this->m_servervars['sv_hostname'];
	  $this->m_srvinfo['iTimelimit'] 	  = $this->m_servervars['timelimit'];
	  $this->m_srvinfo['iFraglimit'] 	  = $this->m_servervars['fraglimit'];
	  $this->m_srvinfo['iMaxClients'] 	  = $this->m_servervars['sv_maxclients'];
	  $this->m_srvinfo['szVersion'] 	  = $this->m_servervars['version'];
	  $this->m_srvinfo['szGameType'] 	  = $this->m_servervars['g_gametype'];
	  $this->m_srvinfo['iPWProtected'] 	  = $this->m_servervars['g_needpass'];
	  $this->m_srvinfo['szServerOS'] 	  = 'N/A';

	  return true;
	}
 }

  // XML::COMMENT (Klasse CCounterStrike festlegen)
  class CCounterStrike
  {
	// XML::COMMENT (Info about players)
	var $m_playerinfo  = "";
	// XML::COMMENT (Info about the server current map, players etc)
	var $m_servervars  = "";
	// XML::COMMENT (Serverrules)
	var $m_serverrules = "";

    // XML::FUNC::timenow ($self->timenow())
    // XML::DESC::timenow (Liefert die genaue Zeit fr das Timeout zurck)
    // XML::VERSION::timenow  (1.0)
    // XML::RETURN::timenow ($doubleval)
	function timenow()
	{
	  return doubleval(ereg_replace('^0\.([0-9]*) ([0-9]*)$','\\2.\\1',microtime()));
	}

    // XML::FUNC::getServerData ($self->getServerData($m_command,$m_serveraddress,$m_port,$m_waittime))
    // XML::DESC::getServerData (Liefert Daten vom Server zurck)
    // XML::VERSION::getServerData  (1.0)
    // XML::RETURN::getServerData ($string)
	function getServerData($m_command,$m_serveraddress,$m_port,$m_waittime)
	{
	  $serverdata    = "";
	  $serverdatalen = 0;

	  if ($m_waittime< 500)
	    $m_waittime= 500;
	  if ($m_waittime>2000)
	    $m_waittime=2000;

	  $m_waittime = doubleval($m_waittime/1000.0);

	  if (!$cssocket=fsockopen("udp://".$m_serveraddress,$m_port,$errnr))
	  {
	    $this->errmsg = "No connection";
	    return "";
	  }

	  socket_set_blocking($cssocket,true);
	  // XML::CODE (socket_set_timeout($cssocket,0,500000);)
	  fwrite($cssocket,$m_command,strlen($m_command));

	  // XML::COMMENT (Mark)
	  $starttime=$this->timenow();
	  do
	  {
	    $serverdata .= fgetc($cssocket);
		$serverdatalen++;
		$socketstatus = socket_get_status($cssocket);
		if ($this->timenow()>($starttime+$m_waittime))
		{
		  $this->errmsg = "Connection timed out";
		  fclose($cssocket);
		  return "";
		}
	  } while ($socketstatus["unread_bytes"] );

	  fclose($cssocket);
	  return $serverdata;
	}

    // XML::FUNC::getNextString ($self->getNextString(&$data)
    // XML::DESC::getNextString (interne Funktion zum Lesen von Daten)
    // XML::VERSION::getNextString  (1.0)
    // XML::RETURN::getNextString ($temp)
	function getNextString(&$data)
	{
	  $temp    = "";
	  $counter = 0;
	  while (ord($data[$counter++])!=0) $temp.=$data[$counter-1];
	  $data=substr($data,strlen($temp)+1);
 	  return $temp;
	}

    // XML::FUNC::getNextByteValue ($self->getNextByteValue(&$data)
    // XML::DESC::getNextByteValue (interne Funktion zum Lesen von Daten)
    // XML::VERSION::getNextByteValue  (1.0)
    // XML::RETURN::getNextByteValue ($temp)
	function getNextByteValue(&$data)
	{
	  $temp = ord($data[0]);
  	  $data = substr($data,1);
  	  return $temp;
	}

    // XML::FUNC::getNextFragValue ($self->getNextFragValue(&$data)
    // XML::DESC::getNextFragValue (interne Funktion zum Lesen von Daten)
    // XML::VERSION::getNextFragValue  (1.0)
    // XML::RETURN::getNextFragValue ($temp)
	function getNextFragValue(&$data)
	{
	  $frags = ord($data[0])+(ord($data[1])<<8)+(ord($data[2])<<16)+(ord($data[3])<<24);
	  if ($frags>=4294967294)
	    $frags-=4294967296;

	  $data=substr($data,4);
	  return $frags;
	}

    // XML::FUNC::getNextPlayTime ($self->getNextPlayTime(&$data)
    // XML::DESC::getNextPlayTime (interne Funktion zum Lesen von Daten)
    // XML::VERSION::getNextPlayTime  (1.0)
    // XML::RETURN::getNextPlayTime ($temp)
	function getNextPlayTime(&$data)
	{
	  $decnumber = ord($data[0])+(ord($data[1])<<8)+(ord($data[2])<<16)+(ord($data[3])<<24);
	  $binnumber = base_convert($decnumber,10,2);

	  while (strlen($binnumber) < 32)
	    $binnumber="0".$binnumber;

	  $exp = abs(base_convert(substr($binnumber,1,8),2,10))-127;

	  if (substr($binnumber,0,1)=="1")
	    $exp=0-$exp;

	  $man=1;
	  $manadd = 0.5;

	  for ($counter=9;$counter<32;$counter++)
	  {
	    if (substr($binnumber,$counter,1)=="1")
	      $man+=$manadd;

		$manadd = $manadd/2;
	  }

	  $time = round(pow(2,$exp)*$man);
	  $playtime = "";

	  if ($time>3600)
	  {
	    $playtime=sprintf("%2dh",$time/3600);
	  }

	  $time %= 3600;
	  $playtime = $playtime.sprintf("%2dm",$time/60);
	  $time %= 60;
	  $playtime = $playtime.sprintf("%2ds",$time);
	  $data = substr($data,5);

	  return $playtime;
	}

    // XML::FUNC::getServerStatus ($self->getServerStatus($m_serveraddress,$m_port,$m_waittime))
    // XML::DESC::getServerStatus (Liefert Status des Servers zurck)
    // XML::VERSION::getServerStatus (1.0)
    // XML::RETURN::getServerStatus (BOOL|$m_servervars|$m_playerinfo)
	function getServerStatus($m_serveraddress,$m_port,$m_waittime)
	{
	  $this->getServerInfo($m_serveraddress,$m_port,$m_waittime);
	  $this->getServerPlayers($m_serveraddress,$m_port,$m_waittime);

	  return true;
	}

	// XML::FUNC::getServerRules ($self->getServerRules($m_serveraddress,$m_port,$m_waittime))
    // XML::DESC::getServerRules (Liefert Rules des Servers zurck)
    // XML::VERSION::getServerRules (1.0)
    // XML::RETURN::getServerRules (BOOL|$m_serverrules)
	function getServerRules($m_serveraddress,$m_port,$m_waittime)
	{
	  $cmd = "\xFF\xFF\xFF\xFFrules\x00";
	  $serverdata = $this->getServerData($cmd,$m_serveraddress,$m_port,$m_waittime);

	  // XML::COMMENT (Check length of returned data, if < 5 something went wrong)
	  if (strlen($serverdata)<5)
	    return false;

	  // XML::COMMENT (Figure out how many rules there are)
	  $rules = (ord($serverdata[5]))+(ord($serverdata[6])*256);

	  if ($rules!=0)
	  {
	    // XML::COMMENT (Strip OOB data)
		$serverdata = substr($serverdata,7);
		for ($i=1;$i<=$rules;$i++)
		{
		  $rulename  = $this->getNextString($serverdata);
		  $rulevalue = $this->getNextString($serverdata);
		  $this->m_serverrules[$rulename] = $rulevalue;
		}

		return true;
	  }
	  else
	  {
	    return false;
	  }
	}

    // XML::FUNC::getServerInfo ($self->getServerInfo($m_serveraddress,$m_port,$m_waittime))
    // XML::DESC::getServerInfo (Liefert Infos des Servers zurck)
    // XML::VERSION::getServerInfo (1.0)
    // XML::RETURN::getServerInfo (BOOL|$m_servervars)
	function getServerInfo($m_serveraddress,$m_port,$m_waittime)
	{
	  $cmd = "\xFF\xFF\xFF\xFFinfo\x00";
	  $serverdata = $this->getServerData($cmd,$m_serveraddress,$m_port,$m_waittime);
	  // XML::COMMENT (Check length of returned data, if < 5 something went wrong)

	  if (strlen($serverdata)<5)
	    return false;

	  // XML::COMMENT (Strip OOB data)
	  $serverdata = substr($serverdata,5);
	  $this->m_servervars["serveraddress"]  = $this->getNextString($serverdata);
	  $this->m_servervars["servername"]		= $this->getNextString($serverdata);
	  $this->m_servervars["mapname"]		= $this->getNextString($serverdata);
	  $this->m_servervars["game"]			= $this->getNextString($serverdata);
	  $this->m_servervars["gamename"]		= $this->getNextString($serverdata);
	  $this->m_servervars["currentplayers"]	= $this->getNextByteValue($serverdata);
	  $this->m_servervars["maxplayers"]		= $this->getNextByteValue($serverdata);
	  $this->m_servervars["protocol"]		= $this->getNextByteValue($serverdata);
	  $this->m_servervars["servertype"]		= $this->getNextByteValue($serverdata);
	  $this->m_servervars["serveros"]		= $this->getNextByteValue($serverdata);

	  // XML::COMMENT (Betriebssystem berprfen)
	  ($this->m_servars["serveros"] == 119) ? ($this->m_servars["serveros"] = "Windows") : ($this->m_servars["serveros"] = "Linux");

	  $this->m_servervars["pwd_protected"]	= $this->getNextByteValue($serverdata);
	  if ($this->m_servervars["is_mod"] = $this->getNextByteValue($serverdata))
      {
	    $this->m_servervars["mod_url"]		= $this->getNextString($serverdata);
	    $this->m_servervars["mod_urldl"]	= $this->getNextString($serverdata);
	  }
	  return true;
	}


    // XML::FUNC::getServerPlayers ($self->getServerPlayers($m_serveraddress,$m_port,$m_waittime))
    // XML::DESC::getServerPlayers (Liefert Spieler auf dem Server zurck)
    // XML::VERSION::getServerPlayers (1.0)
    // XML::RETURN::getServerPlayers (BOOL|$m_playerinfo)
	function getServerPlayers($m_serveraddress,$m_port,$m_waittime)
	{
	  // XML::COMMENT (Servercommand)
	  $cmd = "\xFF\xFF\xFF\xFFplayers\x00";
	  $serverdata = $this->getServerData($cmd,$m_serveraddress,$m_port,$m_waittime);

	  // XML::COMMENT (Check length of returned data, if < 5 something went wrong)
	  if (strlen($serverdata)<5)
	    return false;

	  // XML::COMMENT (Check number of players to read data for)
	  $players = ord($serverdata[5]);

	  // XML::COMMENT (Strip OOB data and other stuff)
	  $serverdata = substr($serverdata,7);
	  for ($i=1;$i<=$players;$i++)
	  {
	    $playername	= htmlspecialchars($this->getNextString($serverdata));
		$frags	    = $this->getNextFragValue($serverdata);
		$playtime	= $this->getNextPlayTime($serverdata);
		$this->m_playerinfo[$i] = array("name"=>$playername,"frags"=>$frags,"time"=>$playtime);
	  }
	  // XML::COMMENT (Sort players in fragorder)
	  if ($players>1)
	    usort($this->m_playerinfo,"fragsort");

	  return true;
	}

    // XML::FUNC::execCmd ($self->execCmd($m_serveraddress,$m_port,$m_waittime,$in_rconpw,$m_port))
    // XML::DESC::execCmd (Fhrt Kommando aus)
    // XML::VERSION::execCmd  (1.0)
    // XML::RETURN::execCmd (BOOL)
	function execCmd($m_serveraddress,$m_port,$m_waittime,$in_rconpw,$m_command)
	{
	  $serverdata    = "";
	  $serverdatalen = 0;

	  if ($m_waittime< 500)
	    $m_waittime= 500;
	  if ($m_waittime>2000)
	    $m_waittime=2000;

	  $m_waittime = doubleval($m_waittime/1000.0);
	  if (!$cssocket=fsockopen("udp://".$m_serveraddress,$m_port,$errnr))
	  {
	    $this->errmsg = "No connection";
	    return "";
	  }

	  socket_set_blocking($cssocket,true);
	  // XML::CODE (socket_set_timeout($cssocket,0,500000);)

	  fwrite($cssocket, sprintf("%c%c%c%c%s%c", 0xFF, 0xFF, 0xFF, 0xFF, "challenge rcon\n\0", 0x00));
	  $me = fread($cssocket, 30);
	  $cid = preg_replace("/challenge rcon (.*)/i","\\1",$me);
	  $cid = preg_replace("/(\D+)(\d+)\n/i","\\2",$cid);
	  fwrite($cssocket, sprintf("%c%c%c%c%s%c", 0xFF, 0xFF, 0xFF, 0xFF, "rcon $cid \"$in_rconpw\" $m_command", 0x00));

      $ans = fread($cssocket,8);
	  (preg_match("/(.*)Ba/i",$ans,$ar)) ? ($ret = 0) : ($ret = 1);

	  if ($ret == 0)
	    return false;
	  else
	    return true;
	}

    // XML::FUNC::convertToStandard ($self->convertToStandard())
    // XML::DESC::convertToStandard (Konvertiert alle Variablen, die in $m_serverinfo gespeichert sind, zu einem gemeinsamen Standard)
    // XML::VERSION::convertToStandard  (1.0)
    // XML::RETURN::convertToStandard (BOOL|$m_srvinfo)
	function convertToStandard()
	{
	  $ar = split(":",$this->m_servarvars["serveraddress"]);

	  $this->m_srvinfo['iServerIP'] 	  = $ar[0];
	  $this->m_srvinfo['iServerPort'] 	  = $ar[1];
	  $this->m_srvinfo['iCurrentClients'] = $this->m_servervars['currentplayers'];
	  $this->m_srvinfo['szMapName'] 	  = $this->m_servervars['mapname'];
	  $this->m_srvinfo['szHostName'] 	  = $this->m_servervars['servername'];
	  $this->m_srvinfo['iTimelimit'] 	  = 'N/A';
	  $this->m_srvinfo['iFraglimit'] 	  = 'N/A';
	  $this->m_srvinfo['iMaxClients'] 	  = $this->m_servervars['maxplayers'];
	  $this->m_srvinfo['szVersion'] 	  = $this->m_servervars['serveros'].$this->m_servervars['servertype'];
	  $this->m_srvinfo['szGameType'] 	  = $this->m_servervars['game'];
	  $this->m_srvinfo['iPWProtected'] 	  = $this->m_servervars['pwd_protected'];

	  return true;
	}
  }

?>
